@@ -1,6 +1,6 @@
 /*
  * Copyright (c) [2011-2015] Novell, Inc.
- * Copyright (c) [2016-2019] SUSE LLC
+ * Copyright (c) [2016-2023] SUSE LLC
  *
  * All Rights Reserved.
  *
@@ -25,13 +25,20 @@
 #define SNAPPER_SNAPSHOT_H
 
 
-#include <time.h>
+#include <ctime>
 #include <sys/types.h>
+#include <cstdint>
 #include <string>
 #include <list>
 #include <map>
 
 #include "snapper/Exception.h"
+#include "snapper/Plugins.h"
+
+
+#ifndef SN_DEPRECATED
+#define SN_DEPRECATED __attribute__((deprecated))
+#endif
 
 
 namespace snapper
@@ -104,8 +111,16 @@ namespace snapper
 	SDir openInfoDir() const;
 	SDir openSnapshotDir() const;
 
+	/**
+	 * Determine iff snapshot is read-only (only for btrfs).
+	 */
 	bool isReadOnly() const;
 
+	/**
+	 * Set snapshot read-only or read-write (only for btrfs).
+	 */
+	void setReadOnly(bool read_only);
+
 	/**
 	 * Determine iff snapshot is default (will be activated on next boot time).
 	 */
@@ -114,7 +129,12 @@ namespace snapper
 	/**
 	 * Change default snapshot (will be activated on next boot time).
 	 */
-	void setDefault() const;
+	void setDefault() const SN_DEPRECATED;
+
+	/**
+	 * Change default snapshot (will be activated on next boot time).
+	 */
+	void setDefault(Plugins::Report& report) const;
 
 	/**
 	 * Determine iff snapshot is active (activated on last boot time).
@@ -145,17 +165,19 @@ namespace snapper
 
 	time_t date;
 
-	uid_t uid;
+	uid_t uid = 0;
 
-	unsigned int pre_num;	// valid only for type=POST
+	bool read_only = true;
+
+	unsigned int pre_num = 0;	// valid only for type=POST
 
 	string description;	// likely empty for type=POST
 	string cleanup;
 	map<string, string> userdata;
 
-	mutable bool mount_checked;
-	mutable bool mount_user_request;
-	mutable unsigned int mount_use_count;
+	mutable bool mount_checked = false;
+	mutable bool mount_user_request = false;
+	mutable unsigned int mount_use_count = 0;
 
 	void writeInfo() const;
 
@@ -163,6 +185,8 @@ namespace snapper
 	void createFilesystemSnapshotOfDefault(bool read_only) const;
 	void deleteFilesystemSnapshot() const;
 
+	void deleteFilelists() const;
+
     };
 
 
@@ -177,8 +201,6 @@ namespace snapper
     {
     public:
 
-	SMD() : description(), cleanup(), userdata() {}
-
 	string description;
 	string cleanup;
 	map<string, string> userdata;
@@ -190,17 +212,15 @@ namespace snapper
     {
     public:
 
-	SCD() : SMD(), read_only(true), empty(false), uid(0) {}
-
-	bool read_only;
+	bool read_only = true;
 
 	/**
 	 * Create an empty snapshot. For btrfs this creates a subvolume
 	 * instead of a snapshot, for other filesystem types ignored.
 	 */
-	bool empty;
+	bool empty = false;
 
-	uid_t uid;
+	uid_t uid = 0;
 
     };
 
@@ -211,7 +231,8 @@ namespace snapper
 
 	friend class Snapper;
 
-	Snapshots(const Snapper* snapper) : snapper(snapper) {}
+	Snapshots(const Snapper* snapper);
+	~Snapshots();
 
 	typedef list<Snapshot>::iterator iterator;
 	typedef list<Snapshot>::const_iterator const_iterator;
@@ -269,18 +290,17 @@ namespace snapper
 
 	void checkUserdata(const map<string, string>& userdata) const;
 
-	iterator createSingleSnapshot(const SCD& scd);
-	iterator createSingleSnapshot(const_iterator parent, const SCD& scd);
-	iterator createSingleSnapshotOfDefault(const SCD& scd);
-	iterator createPreSnapshot(const SCD& scd);
-	iterator createPostSnapshot(const_iterator pre, const SCD& scd);
+	iterator createSingleSnapshot(const SCD& scd, Plugins::Report& report);
+	iterator createSingleSnapshot(const_iterator parent, const SCD& scd, Plugins::Report& report);
+	iterator createSingleSnapshotOfDefault(const SCD& scd, Plugins::Report& report);
+	iterator createPreSnapshot(const SCD& scd, Plugins::Report& report);
+	iterator createPostSnapshot(const_iterator pre, const SCD& scd, Plugins::Report& report);
 
-	iterator createHelper(Snapshot& snapshot, const_iterator parent, bool read_only,
-			      bool empty = false);
+	iterator createHelper(Snapshot& snapshot, const_iterator parent, bool empty, Plugins::Report& report);
 
-	void modifySnapshot(iterator snapshot, const SMD& smd);
+	void modifySnapshot(iterator snapshot, const SMD& smd, Plugins::Report& report);
 
-	void deleteSnapshot(iterator snapshot);
+	void deleteSnapshot(iterator snapshot, Plugins::Report& report);
 
 	unsigned int nextNumber();
 
